<?php
/**
 * @version		1.5.0
 * @package		WebRotate 360 Product Viewer (plugin)
 * @author      WebRotate 360 LLC
 * @copyright	Copyright (C) WebRotate 360 LLC. All rights reserved.
 * @license		GNU General Public License version 2 or later (http://www.gnu.org/copyleft/gpl.html).
 */

defined("_JEXEC") or die;
jimport("joomla.plugin.plugin");

class PlgContentWebRotate360 extends JPlugin
{
    private $viewerWidth;
    private $viewerHeight;
    private $baseWidth;
    private $minHeight;
    private $config;
    private $viewerSkin;
    private $popupSkin;
    private $licensePath;
    private $name;
    private $callback;
    private $graphicsPath;
    private $background;
    private $rootPath;
    private $isPopupGallery;
    private $inBrowserFullscreen = false;
    private $useGoogleEvents;
    private $title;
    private $jDoc;
    private $headerInitialized = false;

    function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);
    }

    public function onContentPrepare($context, &$article, &$params, $page = 0)
    {
        $parts = explode(".", $context);

        $contentMode = $parts[0];

        if (($contentMode !== "com_content")    &&
            ($contentMode !== "mod_k2_content") &&
            ($contentMode !== "com_k2")         &&
            ($contentMode !== "mod_custom")     &&
			($contentMode !== "text")			&&
            ($contentMode !== "com_djcatalog2") &&
            ($contentMode !== "com_virtuemart"))
        {
            return (false);
        }

        if (!$this->headerInitialized)
        {
            $this->initHeaderScripts();
            $this->headerInitialized = true;
        }

        if (isset($article->text))
        {
            $article->text = $this->processCodes($article->text);
        }
    }

    private function initHeaderScripts()
    {
        $this->initHeaderDefaults();
        $this->jDoc = JFactory::getDocument();

        JLoader::import("joomla.version");
        $version = new JVersion();
        if (version_compare($version->RELEASE, "2.5", "<="))
        {
            if (!JFactory::getApplication()->get("jquery"))
            {
                JFactory::getApplication()->set("jquery", true);
                $this->jDoc->addScript(JURI::base() . "plugins/content/webrotate360/imagerotator/html/js/jquery-3.4.1.min.js");
            }
        }
        else
        {
            JHtml::_("jquery.framework");
        }

        $this->jDoc->addScript(JURI::base() . "plugins/content/webrotate360/imagerotator/html/js/imagerotator.js");
        $this->jDoc->addScript(JURI::base() . "plugins/content/webrotate360/prettyphoto/js/jquery.prettyPhoto.js");
        $this->jDoc->addScript(JURI::base() . "plugins/content/webrotate360/webrotate360.js");
/*
        $include_i18n_path = $this->getOption("wr360_i18n_path");
        if (!empty($include_i18n_path))
        {
            $this->jDoc->addScript($include_i18n_path);
        }
*/
        $this->jDoc->addStyleSheet(JURI::base() . "plugins/content/webrotate360/prettyphoto/css/prettyphoto.css");
        $this->jDoc->addStyleSheet(JURI::base() . "plugins/content/webrotate360/imagerotator/html/css/" . $this->viewerSkin . ".css");

        $helperScript = "function getWR360PopupSkin(){return '" . $this->popupSkin . "';}";

        $this->jDoc->addScriptDeclaration($helperScript);
    }

    private function processCodes($content)
    {
        $regex = '/(.?)\[(wr360embed|wr360popup|wr360expand)\b(.*?)(?:(\/))?\](?:(.+?)\[\/\2\])?(.?)/s';

        return (preg_replace_callback($regex, array($this, "replaceCodes"), $content));
    }

    private function replaceCodes($codeMatch)
    {
        $codeId      = $codeMatch[2];
        $codeStr     = $codeMatch[3];
        $codeBefore  = $codeMatch[1];
        $codeAfter   = $codeMatch[6];
        $codeContent = $codeMatch[5];

        $attribs = array();
        if (preg_match_all("/([^,= ]+)=([^,= ]+)/", $codeStr, $attribMatch))
        {
            $attribs = array_combine($attribMatch[1], $attribMatch[2]);
        }

        if (!empty($codeContent))
        {
            if ($codeId === "wr360popup")
                $replacer = $this->popupShortcodeHanlder($attribs, $codeContent);
            elseif ($codeId === "wr360expand")
                $replacer = $this->expandShortcodeHanlder($attribs, $codeContent);
        }
        else
        {
            if ($codeId === "wr360embed")
                $replacer = $this->embedShortcodeHanlder($attribs);
        }

        if (!empty($replacer))
            return ($codeBefore . $replacer . $codeAfter);
        return ("");
    }

    private function expandShortcodeHanlder($attribs, $content)
    {
        $this->initShortcode($attribs);

        $replace  = "";
        $replace .= "<div id='%s' style='cursor:pointer;' class='webrotate360 wr360embed' data-imagerotator='{";
        $replace .= '"onready":"%s", "graphics":"%s", "licfile":"%s", "rootpath":"%s", "xmlfile":"%s", "events":%s, "name":"%s", "fsclick":true, "browserfs":%s';
        $replace .= "}'>";
        $replace .= $content;
        $replace .= "</div>";

        return (sprintf(
            $replace,
            'wr360_' . $this->name,
            $this->callback,
            $this->graphicsPath,
            $this->licensePath,
            $this->rootPath,
            $this->config,
            $this->useGoogleEvents ? "true" : "false",
            $this->name,
            $this->inBrowserFullscreen === "true" ? "true" : "false"));
    }

    private function popupShortcodeHanlder($attribs, $content)
    {
        $this->initShortcode($attribs);

        $replace  = "<a id='%s' title='%s' href='%s?config=%s&amp;root=%s&amp;height=%s&amp;viewname=%s&amp;grphpath=%s&amp;analyt=%s&amp;lic=%s&amp;background=%s&amp;iframe=true&amp;width=%s&amp;height=%s' data-rel=prettyPhoto%s>";
        $replace .= $content;
        $replace .= "</a>";

        return (sprintf(
            $replace,
            'wr360_' . $this->name,
            $this->title,
            JURI::base() . "plugins/content/webrotate360/viewloader_" . $this->viewerSkin . ".html",
            urlencode($this->config),
            urlencode($this->rootPath),
            $this->viewerHeight,
            $this->name,
            urlencode($this->graphicsPath),
            $this->useGoogleEvents ? "true" : "false",
            urlencode($this->licensePath),
            urlencode($this->background),
            str_ireplace("%", "", $this->viewerWidth),
            $this->viewerHeight,
            $this->isPopupGallery ? "[iframe]" : ""
        ));
    }

    private function embedShortcodeHanlder($attribs)
    {
        $this->initShortcode($attribs);

        $replace  = "";
        $replace .= "<div id='%s' class='webrotate360 wr360embed' style='width:%s; height:%s;' data-imagerotator='{";
        $replace .= '"onready":"%s", "graphics":"%s", "licfile":"%s", "rootpath":"%s", "xmlfile":"%s", "basewidth":%s, "events":%s, "name":"%s", "minheight":%s, "background":"%s"';
        $replace .= "}'></div>";

        return (sprintf(
            $replace,
            'wr360_' . $this->name,
            (strpos($this->viewerWidth, '%') === FALSE) ? $this->viewerWidth . 'px' : $this->viewerWidth,
            (strpos($this->viewerHeight, "%") === FALSE) ? $this->viewerHeight . "px" : $this->viewerHeight,
            $this->callback,
            $this->graphicsPath,
            $this->licensePath,
            $this->rootPath,
            $this->config,
            $this->baseWidth,
            $this->useGoogleEvents ? "true" : "false",
            $this->name,
            $this->minHeight,
            $this->background));
    }

    private function initShortcode($attribs)
    {
        $extract = $this->initShortcodeDefaults(
            array(
                "name"         => "",
                "width"        => "",
                "height"       => "",
                "rootpath"     => "",
                "config"       => "",
                "basewidth"    => "",
                "minheight"    => "",
                "gallery"      => "false",
                "viewerskin"   => "",
                "background"   => "",
                "browserfs"    => "",
                "callback"     => "",
                "title"        => ""), $attribs);

        $this->viewerWidth  = str_ireplace("px", "", $this->getOption("wr360_viewer_width"));
        $this->viewerHeight = str_ireplace("px", "", $this->getOption("wr360_viewer_height"));
        $this->baseWidth    = str_ireplace("px", "", $this->getOption("wr360_viewer_basewidth"));
        $this->minHeight    = str_ireplace("px", "", $this->getOption("wr360_viewer_minheight"));
        $this->callback     = $this->getOption("wr360_api_ready_callback");
        $this->config       = $this->getOption("wr360_master_config");
        $this->background   = $this->getOption("wr360_background_color");
        $this->graphicsPath = $this->getOption("wr360_graphics_path");

        if (empty($this->viewerWidth))
            $this->viewerWidth = 300;
        if (empty($this->viewerHeight))
            $this->viewerHeight = 300;
        if (empty($this->baseWidth))
            $this->baseWidth = 0;
        if (empty($this->minHeight))
            $this->minHeight = 0;
        if (empty($this->graphicsPath))
            $this->graphicsPath = JURI::base() . "plugins/content/webrotate360/graphics";

        if (!empty($extract["width"]))
            $this->viewerWidth = str_ireplace("px", "", $extract["width"]);
        if (!empty($extract["height"]))
            $this->viewerHeight = str_ireplace("px", "", $extract["height"]);
        if (!empty($extract["basewidth"]))
            $this->baseWidth = str_ireplace("px", "", $extract["basewidth"]);
        if (!empty($extract["minheight"]))
            $this->minHeight = str_ireplace("px", "", $extract["minheight"]);
        if (!empty($extract["config"]))
            $this->config = $extract["config"];
        if (!empty($extract["viewerskin"]))
            $this->viewerSkin = $extract["viewerskin"];
        if (!empty($extract["callback"]))
            $this->callback = $extract["callback"];
        if (!empty($extract["background"]))
            $this->background = $extract["background"];
        if (!empty($extract["browserfs"]))
            $this->inBrowserFullscreen = $extract["browserfs"];
        if (!empty($extract["title"]))
            $this->title = $extract["title"];

        // Just to keep things short in script.
        if (empty($this->callback))
            $this->callback = "wr360_callback_not_set";

        $this->name = preg_replace('/\s+/', '', $extract["name"]);
        $this->rootPath = $extract["rootpath"];

        $this->isPopupGallery = $extract["gallery"];
        if (!empty($this->isPopupGallery))
            $this->isPopupGallery = ($this->isPopupGallery === "true");
    }

    private function initHeaderDefaults()
    {
        $this->viewerSkin = $this->getOption("wr360_viewer_skin");
        if (empty($this->viewerSkin))
            $this->viewerSkin = "basic";

        $this->popupSkin = $this->getOption("wr360_popup_skin");
        if (empty($this->popupSkin) || ($this->popupSkin == "default"))
            $this->popupSkin = "pp_default";

        $this->licensePath = $this->getOption("wr360_license_path");
        if (empty($this->licensePath))
            $this->licensePath = JURI::base() . "plugins/content/webrotate360/license.lic";

        $this->useGoogleEvents = $this->getOption("wr360_use_googleevents");
        if (!empty($this->useGoogleEvents))
            $this->useGoogleEvents = true;
    }

    private function initShortcodeDefaults($defaults, $attribs)
    {
        $parsed = array();
        foreach($defaults as $name => $value)
        {
            if (array_key_exists($name, $attribs))
            {
                $parsed[$name] = trim($attribs[$name], '"');
                continue;
            }

            $parsed[$name] = $value;
        }

        return ($parsed);
    }

    private function getOption($optionName)
    {
        return ($this->params->get($optionName));
    }
}
